#include <os.h>
#include <nspireio.h>

char *my_strtok(char *string, const char *separator)
{
    char *s1, *s2;
    static char *str_save;
	
    if (string == NULL)
	{
        string = str_save;
        if (string == NULL) return (char *)NULL;
    }
    s1 = string + strspn(string, separator);
    if (*s1 == '\0')
	{
        str_save = NULL;
        return (char *)NULL;
    }
    s2 = strpbrk(s1, separator);
    if (s2 != NULL)
	{
        *s2++ = '\0';
	}
    str_save = s2;

    return s1;
}

BOOL n_ready(void)
{
	volatile unsigned *line_status_reg = is_classic ? (unsigned*)0x90020014 : (unsigned*)0x90020018;
	return *line_status_reg & 0b1;
}

char n_getc(void)
{
	volatile unsigned *line_status_reg = is_classic ? (unsigned*)0x90020014 : (unsigned*)0x90020018;
	volatile unsigned *recv_buffer_reg = (unsigned*)0x90020000;
	while(!(*line_status_reg & 0b1));
	return *recv_buffer_reg;
}

void n_getline(char* dest)
{
	int i = 0;
	while(1)
	{
		char c = n_getc();
		dest[i++] = c;
		if(c == '\n')
		{
			dest[i] = 0;
			break;
		}
	}
}

void n_putc(char c)
{
	volatile unsigned *line_status_reg = is_classic ? (unsigned*)0x90020014 : (unsigned*)0x90020018;
	volatile unsigned *xmit_holding_reg = (unsigned*)0x90020000;
	while(!(*line_status_reg & 0b100000)); // wait for empty xmit holding reg
		*xmit_holding_reg = c;
}

void n_puts(const char *str) {
	while(*str) {
		n_putc(*str++);
	}
}

void n_printf(char *format, ...)
{
	char buf[256];
	memset(buf,'\0',256);
	__builtin_va_list arglist;
	__builtin_va_start(arglist,format);
	vsprintf(buf,format,*(char **)&arglist);
	n_puts(buf);
	__builtin_va_end(arglist);
}

// For now, hardcoded stuff
#define SERVER "irc.prison.net"
#define PORT 6667
#define NICK "nspire"
#define IDENT "compu"
#define REALNAME "TI-Nspire IRC beta test"
#define CHANNEL "#omnimaga"

void getnick(char* src, char* dest)
{
	int nick_end = strcspn(src,"!");
	strncpy(dest,src+1,nick_end-1);
	dest[nick_end-1] = 0;
	//n_printf("DEBUG Nick for(%s): %s\n\n",src,dest);
}
void getcontent(char* src, char* dest)
{
	/*int content_start = strcspn(src+1,":");
	strcpy(dest,src+content_start+2);
	dest[strlen(dest)-1] = 0;*/
	
	// Damn IPv6....
	strcpy(dest,strstr(src," :")+2);
	
	//n_printf("DEBUG Content for (%s): %s\n",src,dest);
}
int main(void)
{
	c_init(0xF,0x0);
	
	// Establish a connection
	n_printf("%s %d\n",SERVER,PORT);
	
	sleep(1000);
	n_printf("NICK %s\n",NICK);
	n_printf("USER %s 0 * :%s\n",IDENT,REALNAME);
	
	while(1)
	{
		if(n_ready())
		{
			// Receive a full line
			char buffer[512];
			char buffer2[512];
			n_getline(buffer);
			strcpy(buffer2,buffer);
			
			char* tokens[64];
			int token_count = 0;
			
			tokens[0] = my_strtok(buffer," \r\n");

			while(tokens[token_count] != NULL)
			{
				tokens[++token_count] = my_strtok(NULL," \r\n");
			}
			
			// Ping, reply with pong
			if(strcmp(tokens[0],"PING") == 0)
			{
				n_printf("PONG %s\n",tokens[1]);
			}
			
			// Welcome message
			else if(strcmp(tokens[1],"001") == 0)
			{
				c_write("Authenticated. Joining channel.\n");
				//sleep(1000);
				n_printf("JOIN %s\n",CHANNEL);
			}
			
			else if(strcmp(tokens[1],"JOIN") == 0)
			{
				char nick[16];
				getnick(tokens[0],nick);
				
				c_swrite("* %s has joined %s\n",nick,tokens[2]+1);
			}
			
			else if(strcmp(tokens[1],"MODE") == 0)
			{
				char nick[16];
				getnick(tokens[0],nick);
				
				c_swrite("* %s sets mode: %s %s\n",nick,tokens[3]+1,tokens[2]);
			}
			
			else if(strcmp(tokens[1],"PART") == 0)
			{
				char nick[16];
				getnick(tokens[0],nick);
				
				c_swrite("* %s has left %s\n",nick,tokens[2]);
			}
			
			else if(strcmp(tokens[1],"QUIT") == 0)
			{
				char nick[16];
				getnick(tokens[0],nick);
				
				char msg[64];
				getcontent(buffer2,msg);
				
				c_swrite("* %s has quit (%s)\n",nick,msg);
			}
			
			// Private / channel message
			else if(strcmp(tokens[1],"PRIVMSG") == 0)
			{
				char nick[16];
				getnick(tokens[0],nick);
				
				//n_printf("DEBUG Buffer2: %s\n\n",buffer2);
				
				char msg[64];
				getcontent(buffer2,msg);
				
				if(strcmp(tokens[3],":\x01VERSION\x01") == 0)
				{
					n_printf("NOTICE %s :\x01VERSION NspIRC 0.1 Alpha\x01\n",nick);
				}
				else if(strcmp(tokens[2],CHANNEL) == 0)
				{
					c_swrite("<%s> %s\n",nick,msg);
				}
				else if(strcmp(tokens[2],NICK) == 0)
				{
					c_swrite("** PM <%s> %s\n",nick,msg);
				}
			}
			
			//c_writec(n_getc());
		}
		if(any_key_pressed())
		{
			if(isKeyPressed(KEY_NSPIRE_ESC))
			{
				// Exit
				n_puts("QUIT :ESC key pressed\n");
				return 0;
			}
			
			if(isKeyPressed(KEY_NSPIRE_EE))
			{
				n_printf("PRIVMSG %s :",CHANNEL);
			}
			
			// Otherwise assume it is a char
			n_putc(c_readc());
			wait_no_key_pressed();
		}
		idle();
	}
	
	return 0;
}
